@@ -15,17 +15,19 @@ import click
 
 import omnizart.constants.datasets as dset
 from omnizart import MODULE_PATH
-from omnizart.remote import download, download_large_file_from_google_drive
+from omnizart.remote import download_large_file_from_google_drive
 from omnizart.utils import ensure_path_exists, synth_midi
 from omnizart.constants.midi import SOUNDFONT_PATH
 from omnizart.cli.music import music
 from omnizart.cli.drum import drum
 from omnizart.cli.chord import chord
+from omnizart.cli.vocal import vocal
+from omnizart.cli.vocal_contour import vocal_contour
 from omnizart.cli.transcribe import transcribe
 
 
 SUB_COMMAND_GROUP = [
-    {"Transcription": ["music", "chord", "drum", "transcribe"]},
+    {"Transcription": ["music", "chord", "drum", "vocal", "vocal-contour", "transcribe"]},
     {"Utilities": ["download-checkpoints", "download-dataset", "synth"]}
 ]
 
@@ -72,38 +74,37 @@ def entry():
 
 @click.command()
 @click.argument(
-    "dataset", type=click.Choice(["Maestro", "MusicNet", "McGill", "BPS-FH", "Ext-Su"], case_sensitive=False)
+    "dataset",
+    type=click.Choice(["Maestro", "MusicNet", "McGill", "BPS-FH", "Ext-Su", "MIR1K", "CMedia"], case_sensitive=False)
 )
 @click.option(
     "-o", "--output", default="./", help="Path for saving the downloaded dataset.", type=click.Path(writable=True)
 )
-@click.option("--unzip", help="Whether to unzip the downloaded dataset", is_flag=True)
-def download_dataset(dataset, output, unzip):
+def download_dataset(dataset, output):
     """A quick command for downloading datasets."""
-    url = {
-        "maestro": dset.MaestroStructure.url,
-        "musicnet": dset.MusicNetStructure.url,
-        "mcgill": dset.McGillBillBoard.url,
-        "bps-fh": dset.BeethovenSonatas.url,
-        "ext-su": dset.ExtSuStructure.url
+    struct = {
+        "maestro": dset.MaestroStructure,
+        "musicnet": dset.MusicNetStructure,
+        "mcgill": dset.McGillBillBoard,
+        "bps-fh": dset.BeethovenSonatasStructure,
+        "ext-su": dset.ExtSuStructure,
+        "mir1k": dset.MIR1KStructure,
+        "cmedia": dset.CMediaStructure
     }[dataset.lower()]
-    ensure_path_exists(output)
     click.echo(f"Downloading {dataset} dataset and save to {output}")
-    if "drive.google.com" in url:
-        download_large_file_from_google_drive(url, save_path=output, save_name=dataset + ".zip", unzip=unzip)
-    else:
-        download(url, save_path=output, unzip=unzip)
+    struct.download(save_path=output)
 
 
 @click.command()
 @click.option("--output-path", help="Explicitly specify the path to the omnizart project for storing checkpoints.")
 def download_checkpoints(output_path):
-    """Downlaod the archived checkpoints of different models."""
+    """Download the archived checkpoints of different models."""
     CHECKPOINTS = {
         "chord_v1": {
-            "fid": "1QX5bBoYzZyC2fvK26YEtF_Hqt3DzhiHk",
-            "save_as": "checkpoints/chord/chord_v1/weights.data-00000-of-00001",
-            "file_length": 132717707
+            "fid": "1sz83HC_bkA0Gp9G0TtX7cy3jKkANbR8R",
+            "save_as": "checkpoints/chord/chord_v1.zip",
+            "file_length": 87005425,
+            "unzip": True
         },
         "drum_keras": {
             "fid": "1seqz_pi20zB8rq1YJE0Jbk1SwkJ9hOCK",
@@ -113,7 +114,7 @@ def download_checkpoints(output_path):
         "music_pop": {
             "fid": "1-kM27jR_iCvF8Z-3pAFG-nrMRktyxxJ0",
             "save_as": "checkpoints/music/music_pop/weights.h5",
-            "file_length": 33813824
+            "file_length": 33820096
         },
         "music_piano": {
             "fid": "1x9_qjXSiM4GAxpvKfdYJK5S3SLdlCl2I",
@@ -124,6 +125,17 @@ def download_checkpoints(output_path):
             "fid": "18IqdrR3IhFP52H6w1HgTlPQUJXKHXm9u",
             "save_as": "checkpoints/music/music_note_stream/weights.h5",
             "file_length": 33816384
+        },
+        "vocal_semi": {
+            "fid": "1F3_qns6jc7MjDYigNsKQLJeawsAWAq_x",
+            "save_as": "checkpoints/vocal/vocal_semi.zip",
+            "file_length": 213112805,
+            "unzip": True
+        },
+        "vocal_contour": {
+            "fid": "1w0k0wioN8dnf63E0RgdLB8r-FeBnhwJt",
+            "save_as": "checkpoints/vocal/contour/weights.h5",
+            "file_length": 50732192
         }
     }
 
@@ -138,9 +150,12 @@ def download_checkpoints(output_path):
         save_name = os.path.basename(info["save_as"])
         save_path = os.path.dirname(info["save_as"])
         save_path = os.path.join(output_path, save_path)
+        unzip = info.get("unzip", False)
         download_large_file_from_google_drive(
-            info["fid"], file_length=info["file_length"], save_path=save_path, save_name=save_name
+            info["fid"], file_length=info["file_length"], save_path=save_path, save_name=save_name, unzip=unzip
         )
+        if unzip:
+            os.remove(os.path.join(save_path, save_name))
 
 
 @click.command()
@@ -188,6 +203,8 @@ def synth(input_midi, output_path, sf2_path):
 entry.add_command(music)
 entry.add_command(drum)
 entry.add_command(chord)
+entry.add_command(vocal)
+entry.add_command(vocal_contour)
 entry.add_command(transcribe)
 entry.add_command(download_dataset)
 entry.add_command(download_checkpoints)
