@@ -3,15 +3,12 @@
 import os
 import re
 import types
-import pickle
 import logging
 import uuid
 import concurrent.futures
 import importlib
 from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
 
-import yaml
-import librosa
 import jsonschema
 import pretty_midi
 import scipy.io.wavfile as wave
@@ -72,72 +69,6 @@ def get_logger(name=None, level="warn"):
 logger = get_logger("Omnizart Utils")
 
 
-def dump_pickle(data, save_to):
-    """Dump data to the given path.
-
-    Parameters
-    ----------
-    data: python objects
-        Data to store. Should be python built-in types like `dict`, `list`, `str`, `int`, etc
-    save_to: Path
-        The full path to store the pickle file, including file name.
-        Will create the directory if the given path doesn't exist.
-
-    """
-    base_dir = os.path.dirname(save_to)
-    ensure_path_exists(base_dir)
-    with open(save_to, "wb") as pkl_file:
-        pickle.dump(data, pkl_file)
-
-
-def load_pickle(pickle_file):
-    """Load pickle file from the given path
-
-    Read and returns the data from the given pickle file path
-
-    Parameters
-    ----------
-    pickle_file: Path
-        The full path to the pickle file for read
-
-    Returns
-    -------
-    object
-        Python object, could be `dict`, `list`, `str`, etc.
-    """
-    return pickle.load(open(pickle_file, "rb"))
-
-
-def load_audio_with_librosa(audio_path, sampling_rate=44100):
-    """Load audio from the given path with librosa.load
-
-    Parameters
-    ----------
-    audio_path: Path
-        Path to the audio.
-    sampling_rate: int
-        Target sampling rate after loaded.
-
-    Returns
-    -------
-    audio: 1D numpy array
-        Raw data of the audio.
-    tar_sampling_rate: int
-        Sampling rate of the audio. Will be the same as the given ``sampling_rate``.
-    """
-    return librosa.load(audio_path, mono=True, sr=sampling_rate)
-
-
-def load_yaml(yaml_path):
-    return yaml.load(open(yaml_path, "r"), Loader=yaml.Loader)
-
-
-def write_yaml(json_obj, output_path, dump=True):
-    # If dump is false, then the json_obj should be yaml string already.
-    out_str = yaml.dump(json_obj) if dump else json_obj
-    open(output_path, "w").write(out_str)
-
-
 def camel_to_snake(string):
     """Convert a camel case to snake case"""
     return re.sub(r"(?<!^)(?=[A-Z])", "_", string).lower().replace("__", "_")
@@ -299,7 +230,7 @@ def ensure_path_exists(path):
         os.makedirs(path)
 
 
-def parallel_generator(func, input_list, max_workers=2, use_thread=False, chunk_size=None, timeout=300, **kwargs):
+def parallel_generator(func, input_list, max_workers=2, use_thread=False, chunk_size=None, timeout=600, **kwargs):
     if chunk_size is not None and max_workers > chunk_size:
         logger.warning(
             "Chunk size should larger than the maximum number of workers, or the parallel computation "
@@ -350,6 +281,16 @@ def synth_midi(midi_path, output_path, sampling_rate=44100, sf2_path=SOUNDFONT_P
     wave.write(output_path, sampling_rate, raw_wav)
 
 
+def resolve_dataset_type(dataset_path, keywords):
+    low_path = os.path.basename(os.path.abspath(dataset_path)).lower()
+    d_type = [val for key, val in keywords.items() if key in low_path]
+    if len(d_type) == 0:
+        return None
+
+    assert len(set(d_type)) == 1
+    return d_type[0]
+
+
 class LazyLoader(types.ModuleType):
     """Lazily import a module, mainly to avoid pulling in large dependencies.
 
@@ -389,3 +330,8 @@ class LazyLoader(types.ModuleType):
     def __dir__(self):
         module = self._load()
         return dir(module)
+
+
+def get_filename(path):
+    abspath = os.path.abspath(path)
+    return os.path.splitext(os.path.basename(abspath))[0]
